function [ model, xspecs, testspecs ] = wfmm_setup( Y, X, Z, MCMCspecs, wavespecsx, pca, testspecs, pcaspecsx)
%function [ model ] = wfmm_setup( Y, X, Z, MCMCspecs, wavespecsx, pcaspecsx, TSspecs)
%WFMM_SETUP Builds the design matrix for performing function-on-function
%           regression as in Meyer et al. 2014 (Biometrics) using the wfmm
%           exectuable file availble from the Biostatistic Group at MD
%           Anderson
%
%   Created: 7/18/2014
%   By: Mark John Meyer

    % assume one sample if not otherwise specified %
    if nargin < 7;
        testspecs.twosample           = 0;
    end;
    
    % default to PCA-based approached %
    if nargin < 6;
        pca = 1;
    end;

    %% check and assign two sample specs %%
    twosample   = testspecs.twosample;
    if twosample == 1 && length(fieldnames(testspecs)) < 2;
        error('Grouping variable required for two sample model');
    end;
    if twosample == 1;
        group   = testspecs.group;
    end;
    
    %% center and scale X %%
    N           = size(X,1);
    t           = size(X,2);
    scaleX      = zeros(N,t);
    for i = 1:t,
        time_i          = X(:,i);
        tempStd         = std(time_i);
        tempMean        = mean(time_i);
        scaleX(:,i)     = (time_i-tempMean)/tempStd;
    end;

    %% check if random effects exist set sampleu accordingly %%
    if isempty(Z);
        MCMCspecs.sampleU           = 0;
    else
        MCMCspecs.sampleU           = 1;
    end;
    
    %% extract alph levels and xticks for compression %%
    alph                    = wavespecsx.alph;
    xticks                  = wavespecsx.xticks;
    
    %% compression performed as in Morris et al. () %%
    [compRes, compSet, D_all, C, keep, wavespecsc]  = Wavelet_compress_v1208_rectangular(scaleX,wavespecsx,1,alph,xticks);

    %% select compression level %%
    comp_level              = wavespecsx.comp_level;
    
    %% additional compression output %%
    compOut.compRes         = compRes;
    compOut.compSet         = compSet;
    compOut.C               = C;

    %% update wavespecs %%
    % D is compressed, wavelet transformed data
    wavespecsc.compOut      = compOut;
    wavespecsc.keep         = keep(alph == comp_level,:);
    wavespecsc.compress     = 1;
    D                       = D_all(:,wavespecsc.keep == 1);

    %% get new Kj %%
    [wavespecsc.Kj_comp, wavespecsc.Kj_all] = get_Kj_compress(wavespecsc);
    wavespecsc.Kj_all       = wavespecsc.Kj_all';
    wavespecsc.J_all        = length(wavespecsc.Kj_all);
    wavespecsc.J            = length(wavespecsc.Kj_comp);
    wavespecsc.K            = sum(wavespecsc.Kj_comp);
    
    %% perform PCA on X after DWT %%
    if pca == 1;
        % Default PCA specs for X-space
        if nargin < 8;
            pcaspecsx.pca_cutoff    = [0.9, 0.95, 0.975, 0.99, 0.995, 0.999, 0.9999];
            pcaspecsx.pca_level     = 0.99;
        end;
        pca_cutoff              = pcaspecsx.pca_cutoff;
        [coef, score, eigen]    = princomp(D);
        comp_var                = cumsum(eigen)./sum(eigen);
        pca_keep                = zeros(size(pca_cutoff,2),1);
        for i = 1:size(pca_cutoff,2),
            pca_keep(i)         = find(comp_var > pca_cutoff(i), 1);
        end;
        pca_out                 = [pca_cutoff' pca_keep];

        %% select PCA level %%
        pca_level               = pcaspecsx.pca_level;

        %% select PCs to use %%
        X                       = score(:,1:pca_out(pca_out(:,1) == pca_level,2));

        %% generate column mean matrix %%
        col_means               = mean(D);
        col_mean_mat            = zeros(t,size(D,2));
        for i = 1:t;
            col_mean_mat(i,:)   = col_means;
        end;

        %% save PCA specs %%
        pcaspecsx.X             = X;
        pcaspecsx.pca           = 1;
        pcaspecsx.score         = score;
        pcaspecsx.coef          = coef;
        pcaspecsx.eigen         = eigen;
        pcaspecsx.mean_mat      = col_mean_mat;
        pcaspecsx.output        = pca_out;
    else
        pcaspecsx.pca           = 0;
    end;
    
    %% twosample design matrix %%
    if twosample == 1;
        if pca == 1;
            %% set up big design matrix %%
            X0          = X(group == 0,:);
            X1          = X(group == 1,:);
            twoSize     = size(X0,2) + size(X1,2);
            X           = zeros(N, twoSize);

            %% fill in big design matrix %%
            X(group == 0, 1:size(X0,2))                 = X0;
            X(group == 1, (size(X0,2)+1):(twoSize))     = X1;            
        else
            %% set up big design matrix %%
            tsX         = zeros(N, 2*size(D,2));
            group0      = find(group == 0);
            group1      = find(group == 1);

            %% fill in big design matrix %%
            tsX(group0, 1:size(D,2))                    = D(group0, :);
            tsX(group1, (size(D,2)+1):(2*size(D,2)))    = D(group1, :);
            
            %% assign tsX to D
            D           = tsX;
        end;
    end;

    %% model Set up %%
    if twosample == 1;
        %% allows for group-specific intercepts %%
        a0  = zeros(N,1);
        a1  = zeros(N,1);
        a0(group == 0)  = 1;
        a1(group == 1)  = 1;
        if gbf == 1;
            model.X     = [a0 a1 X];
        else
            model.X     = [a0 a1 D];
        end;
    else
        %% include intercept %%
        a   = ones(N,1);
        if pca == 1;
            model.X     = [a X];
        else
            model.X     = [a D];
        end;
    end;
    model.Z{1}  = Z;
    model.C     = ones(size(Y,1),1);
%     model.m     = 1; % edit for more levels of random effects
%     model.H     = 1;
%     model.Hstar = 0;

    %% set up FDR for post-processing %%
    FDR     = testspecs.FDR;
    if length(FDR) == 1;
        testspecs.delt  = FDR(1);
        testspecs.alf   = 0.05;
    end;
    if length(FDR) == 2;
        testspecs.delt  = FDR(1);
        testspecs.alf   = FDR(2);
    end;
    if length(FDR) == 3;
        testspecs.delt  = FDR(1:2);
        testspecs.alf   = FDR(3);
    end;
    if length(FDR) > 3;
        error('Too many FDR inputs');
    end;
    
    %% function output %%
    xspecs.wavespecsc   = wavespecsc;
    xspecs.pcaspecs     = pcaspecsx;

end

