%%%%%%%%%%%%%%%%%%%%%%%%%%% Example Code %%%%%%%%%%%%%%%%%%%%%%%%%%%
% Example code for Bayesian Function-on-Function Regression for    %
%  Multi-Level Functional Data                                     %
% By Meyer, MJ, Coull, BA, Versace, F, Cinciripini, P, and Morris, %
%  JS.                                                             %
%                                                                  %
% Simulated RIDGE Surf, burnin 1000, sample 1000                   %
% x(v) ~ GP(0, S) S AR(1) estimated covariance from data           %
% Sample tau and pi                                                %
% Keep PCA 99.00%, N = 200                                         %
%                                                                  %
% Created:      7/16/2014                                          %
% Modified:     7/23/2014                                          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% addpaths %%
addpath('./CODE')

%% set seed (if you want to) %%
% seeds used for simualtion: 1 to 200
rng(1);

%% data parameters %%
N       = 200;
T       = 225;

%% grid for simulation surfaces %%
[v, t]  = meshgrid(0:1:(T-1));

%% scale to control STNR %%
stnrs   = 1.75;

%% Simulate Coefficients %%
b1var   = 0.003;
b1      = stnrs*(1/(125*sqrt(2*pi*b1var)))*exp(-1/(2*b1var)*(t./T-v./T).^2);

%% plot ridge (if you want to) %%
% zmin    = -0.03;
% zmax    = 0.11;
% Xaxis   = [0 (225-1)];
% Yaxis   = [0 (225-1)];
% figure
% colormap(hot)
% imagesc(b1,[zmin,zmax])
% colorbar
% % set(gca,'Ydir','reverse','Xdir','reverse')
% set(gca,'FontSize',14)
% set(gca,'YTick',[1,75:75:225],'YTickLabel',[225,150,75,0])
% set(gca,'XTick',[1,75:75:225],'XTickLabel',[0,75,150,225])
% % set(hcb,'YTick',0:0.02:0.1,'FontSize',14)
% xlabel('v','FontSize',14)
% ylabel('t','FontSize',14)
% title('Ridge + Peak','FontSize',14)

%% generate AR(1) covariance matrix %%
ar1Corr     = eye(T);
sigma       = 4.0835;
rho         = 0.9917;
for i = 1:T,
    for j = (i+1):T,
        ar1Corr(i,j) = rho^(j-i);
        ar1Corr(j,i) = rho^(j-i);
    end;
end;

ar1Cov      = sigma*ar1Corr;

%% generate x(v) based on AR(1) cov %%
simX        = NaN(N,T);
muX         = zeros(T,1);
for i = 1:N;
    simX(i,:)   = mvnrnd(muX,ar1Cov);
end;

%% select repeated measures %%
simid       = sort([1:(N/2) 1:(N/2)]);

%% parameters for simulated data %%
NS          = size(simX,1);
TS          = size(simX,2);
nS          = size(unique(simid),2);

%% build Z %%
Z           = zeros(N,nS);
subjects    = unique(simid);
for i = 1:length(unique(simid));
    subi        = subjects(i);
    rowid       = find(simid == subi);
    onev        = ones(length(rowid),1);
    Z(rowid,i)  = onev;
end;

%% scale and center simX so var = 1 at each time point %%
scaleX      = zeros(NS,TS);
for i = 1:TS,
    time_i          = simX(:,i);
    tempStd         = std(time_i);
    tempMean        = mean(time_i);
    scaleX(:,i)     = (time_i-tempMean)/tempStd;
end;

%% generate covariance structure for E ~ GP(0, Sigma_E) %%
CovStr      = eye(TS);
sigmae      = 0.1;
rho         = 0.5;
for i = 1:TS,
    for j = (i+1):TS,
        CovStr(i,j) = rho^(j-i);
        CovStr(j,i) = rho^(j-i);
    end;
end;

Sigma_E     = sigmae*CovStr;

%% use Sigma_E to generate matrix of model errors %%
E           = zeros(NS,TS);
muE         = zeros(TS,1);
for i = 1:NS,
    E(i,:)  = mvnrnd(muE,Sigma_E);
end;

%% generate covariance structure for Ui ~ GP(0, Sigma_Ui) %%
CovStr      = eye(TS);
sigmau      = sigmae/2;
rho         = 0.75;
for i = 1:TS,
    for j = (i+1):TS,
        CovStr(i,j) = rho^(j-i);
        CovStr(j,i) = rho^(j-i);
    end;
end;

Sigma_Ui    = sigmau*CovStr;

%% use Sigma_Ui to generate matrix of subject specific errors %%
U           = zeros(nS,TS);
muU         = zeros(TS,1);
for i = 1:nS,
    U(i,:)  = mvnrnd(muU,Sigma_Ui);
end;

%% set model params %%
testspecs.twosample   = 0;
if testspecs.twosample == 0;
    testspecs.FDR       = [0.05, 0.05];
else
    testspecs.FDR       = [0.05, 0.05, 0.05];
    testspecs.group     = []; % grouping variable for two sample, not needed for this example
end

%% x(v) wavelet specs %%
wavespecsx.comp_level       = 0.99;
wavespecsx.alph             = [0.9,0.95,0.99,0.995,0.999,0.9999];
wavespecsx.xticks           = [5 10 15 20 25 30 35 40 45 50 55];
wavespecsx.nlevels          = 3;
wavespecsx.wavelet          = 'db4';
wavespecsx.wtmode           = 'zpd';
wavespecsx.ndim             = 1;

%% y(t) wavelet specs %%
wavespecs.nlevels           = 3;
wavespecs.wavelet           = 'd4';     % use daubechies with 4 vanishing moments
wavespecs.boundary          = 'zero';   % use zero padding

%% MCMC specs %%
MCMCspecs.B                     = 1000;
MCMCspecs.burnin                = 1000;
MCMCspecs.thin                  = 1;
MCMCspecs.propvar_omega         = 1;
MCMCspecs.nj_nosmooth           = 1;        % can be 0, if is 0, don't do any constraint for pi_{ij}, bigTau_{ij}.
MCMCspecs.minp                  = 1e-14;
MCMCspecs.minT                  = 1;
MCMCspecs.bigT                  = 1000;
MCMCspecs.maxO                  = 1e20;
MCMCspecs.minVC                 = 1e-20;
MCMCspecs.VC0_thresh            = 1e-4;
MCMCspecs.delta_omega           = 1e-4;
MCMCspecs.omega_MOM_maxiter     = 100;
MCMCspecs.omega_MOM_convcrit    = 1e-3;
MCMCspecs.time_update           = 100;
MCMCspecs.missing_data          = 0;
% MCMCspecs.update_pi_tau         = 1;        % update pi and tau (1), use EB estiamtes (0)
% MCMCspecs.tau_prior_var         = 1e3;      % the variance of tau_{ijk} when finding prior parameters for tau_{ijk}.
% MCMCspecs.pi_prior_var          = 0.06;     % this range should be in [0.02 0.09].

%% set up post process specs %%
%  NOTE: the default value is taken if field is left out
% PostProcessSpecs.L          = [];   % contrast matrix, specifies linear combinations of effects
% PostProcessSpecs.quantiles  = [0.5,1,2.5,5,95,97.5,99,99.5];   % specifies quantiles
% PostProcessSpecs.effect_size    = [0.3219, 0.585, 0.8074, 1];   % FDR
% values for WFMM, i.e. not in correct space for FFR inference
PostProcessSpecs.keep_beta_samples  = 1;    % keep samples of coef
PostProcessSpecs.compute_U          = 1;    % compute random effects (1), don't (0)
PostProcessSpecs.keep_U_samples     = 0;    % keep samples of random effects (1), don't (0)

%% generate Y %%
Y               = scaleX*b1 + Z*U + E; % true intercept assumed zero
X               = scaleX;

%% Center and Scale Y by time (not needed for simulated data) %%
% scaleY      = zeros(N,t);
% for i = 1:t,
%     time_i          = Y(:,i);
%     tempStd         = std(time_i);
%     tempMean        = mean(time_i);
%     scaleY(:,i)     = (time_i-tempMean)/tempStd;
% end;
% Y           = scaleY;

%% prep inputs for WFMM %%
% [ model ] = wfmm_setup( Y, X, Z, MCMCspecs, wavespecsx, pca, TSspecs, pcaspecsx);   % pcaspecs are defaulted in wfmm_setup
[ model, xspecs, testspecs ] = wfmm_setup( Y, X, Z, MCMCspecs, wavespecsx, 1, testspecs);    % model includes design matrix with intercept

%% save inputs for WFMM executable %%
save('inputs', 'Y', 'model', 'wavespecs', 'MCMCspecs', 'PostProcessSpecs');

%% OR run from MATLAB?? %%
! ./wfmm inputs.mat outputs.mat > wfmmlog.log

%% wfmm finished %%
fprintf('\n Done with WFMM \n')

%% load output file %%
% cd('/Users/markmeyer/Dropbox/Research/Publications/Biometrics/December 2013/Revision/Computing/Script')
out = load('outputs.mat');

%% load results for post-processing %%
% addpath('/Volumes/homes/home07/mmeyer/PAPER1/ANALYSIS/Script/CODE')
[beta, fullfilename]    = readbeta(out.MCMCspecs.B, size(out.model.X,2), out.wavespecs.T, 'outputs_beta.dat');

%% run post-processor %%
[ postout ]             = PostProcess_beta(beta,model,xspecs.wavespecsc,out.wavespecs,xspecs.pcaspecs,testspecs);

%% save post-processed results %%
save('postProcess','postout');





